module namespace topic = "http://www.architextus.com/xquery/library/how2_topics";
import module namespace app = "http://www.architextus.com/xquery/library/how2_appvars" at "How2_AppVars.xqm";
import module namespace content-refs = "http://www.architextus.com/xquery/library/how2_contentrefs" at "How2_ContentRefs.xqm";
import module namespace filter = "http://www.architextus.com/xquery/library/how2_filter" at "How2_Filter.xqm";
import module namespace menu = "http://www.architextus.com/xquery/library/how2_menus" at "How2_Menus.xqm";

declare namespace rest="http://exquery.org/ns/restxq";

declare %rest:path("/how2-xml-topic/{$id}")
   %rest:GET
   %rest:query-param("lang", "{$lang}", 'en-us')
   function topic:xml-get-how2-topic($id as xs:string, $lang as xs:string) as node()*{
    for $x in app:open-db($lang)//*[self::media or self::topic][@id=$id]
    return $x
};

declare %rest:path("/how2-filtered-xml-topic/{$menu-id}/{$id}")
   %rest:GET
   %rest:query-param("application", "{$application}", '')
   %rest:query-param("carrier", "{$carrier}", '')
   %rest:query-param("country", "{$country}", '')
   %rest:query-param("lang", "{$lang}", 'en-us')
   %rest:query-param("model", "{$model}", '')
   %rest:query-param("os", "{$os}", '')
   %rest:query-param("static", "{$static}", '')
   function topic:xml-get-how2-filtered-topic($id as xs:string, $menu-id as xs:string, 
                               $application as xs:string, $carrier as xs:string, $country as xs:string, 
                               $lang as xs:string, $model as xs:string, $os as xs:string, $static as xs:string) as node()*{

      let $x := topic:xml-get-how2-topic($id, $lang)
      let $filtered-menu := menu:xml-filtered-get-menu($menu-id, $menu-id,
                               $application, $carrier, $country, $lang,
                               $model, $os, '', $static)
      let $prompted-content := content-refs:resolve-references($x, $filtered-menu, $lang, $static)
      (:let $debug := file:write('debug-topic-with-video1.xml', $prompted-content)   :)
      let $filtered-content := filter:filter-node($prompted-content, $application, $carrier, $country, 
                              $lang, $model, $os)

     return if (count($filtered-content//node()) = 0) 
            then <empty/>
            else $filtered-content
};

declare %rest:path("/how2-html-topic/{$menu-id}/{$id}")
   %rest:GET
   %rest:query-param("application", "{$application}", '')
   %rest:query-param("carrier", "{$carrier}", '')
   %rest:query-param("country", "{$country}", '')
   %rest:query-param("keywords", "{$keywords}, ''")
   %rest:query-param("lang", "{$lang}", 'en-us')
   %rest:query-param("model", "{$model}", '')
   %rest:query-param("os", "{$os}", '')
   %rest:query-param("submenu-id", "{$submenu-id}", '0')
   %rest:query-param("static", "{$static}", 'no')
   %rest:query-param("standalone", "{$standalone}", 'no')
   %output:indent("no")
   %output:method("html")
   %output:html-version("5.0")
   function topic:html-get-how2-topic($id as xs:string, $menu-id as xs:string, $application as xs:string,
                               $carrier as xs:string, $country as xs:string, $lang as xs:string,
                               $model as xs:string, $os as xs:string, $keywords as xs:string, $submenu-id as xs:string, $static as xs:string, $standalone as xs:string) as node()*{

    let $filtered-content := topic:xml-get-how2-filtered-topic($id, $menu-id, $application, $carrier, $country, $lang, $model, $os, $static)
     
    (: Transform to HTML :)
    let $stylesheet := doc(app:get-topic2html-xsl())
    let $html-params := map{ "menu-id" := $menu-id, "application" := $application, 
                        "carrier" := $carrier, "country" := $country, "lang" := $lang, "model" := $model,
                        "os" := $os, 'keywords' := $keywords, 'submenu-id' := $submenu-id, "rest-path" := $app:basex_rest_path,
                    "restxq-path" := $app:basex_restxq_path, "static" := $static, "standalone" := $standalone}
    (:return $filtered-content:)
    return xslt:transform($filtered-content, $stylesheet, $html-params)
};

declare %rest:path("/how2-html-topic-title-content/{$id}")
   %rest:GET
   %rest:query-param("application", "{$application}", '')
   %rest:query-param("carrier", "{$carrier}", '')
   %rest:query-param("country", "{$country}", '')
   %rest:query-param("lang", "{$lang}", 'en-us')
   %rest:query-param("model", "{$model}", '')
   %rest:query-param("os", "{$os}", '')
   %rest:query-param("submenu-id", "{$submenu-id}", '0')
   %rest:query-param("static", "{$static}", 'no')
   function topic:html-get-how2-topic-title-content($id as xs:string, $application as xs:string,
                               $carrier as xs:string, $country as xs:string, $lang as xs:string,
                               $model as xs:string, $os as xs:string, $submenu-id as xs:string, $static as xs:string) as node()*{
    (: Should be only one heading, but d0e125 has more... please split further :)
    (: Heading should be mandatory, but d0e818 is missing it... with DTD for split content, we could validate upon upload to the database :)
    let $x := if (topic:xml-get-how2-topic($id, $lang)/heading[1]) then <heading-line>{topic:xml-get-how2-topic($id, $lang)/heading[1]/node()}</heading-line>
                                                            else <heading-line>[Oups! No title!]</heading-line>

    (: No need for menu context, always grabbing the title :)
    let $prompted-content := content-refs:resolve-references($x, <null/>, $lang, $static)
    let $filtered-content := filter:filter-node($prompted-content, $application, $carrier, $country, 
                              $lang, $model, $os)
     
    (: Transform to HTML :)
    let $stylesheet := doc(app:get-topic2html-xsl())
    let $html-params := map{"application" := $application, 
                        "carrier" := $carrier, "country" := $country, "lang" := $lang, "model" := $model,
                        "os" := $os, "submenu-id" := $submenu-id, "rest-path" := $app:basex_rest_path,
                        "restxq-path" := $app:basex_restxq_path, "static" := $static}
    
    (:return $filtered-content:)
    return xslt:transform($filtered-content, $stylesheet, $html-params)
};
